#!/bin/env python

#######################################################################
# Copyright (C) 2019 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################


def pb_region_paths(overlap_map):
    """Uses overlap information to build a directional, acyclic graph that contains all linear paths

    :param overlap_map: contains overlap information
    :return: graph
    """
    graph = {}
    reached_list = {}
    path_count = {'END': 1}
    for feature in overlap_map:
        reached = []
        links = []
        if not feature[0] == 'END':
            path_count[feature[0]] = 0
        for candidate in feature[1]:
            links.append(candidate)
            path_count[feature[0]] += path_count[candidate]
            reached.append(candidate)
            for i in reached_list[candidate]:
                if i in feature[1]:
                    feature[1].remove(i)
                if i not in reached:
                    reached.append(i)
        reached_list[feature[0]] = reached
        graph[feature[0]] = links
    return graph, path_count['START']


def pb_region_mapper(overlap_region, features, max_overlap, max_overlap_percentage):
    """ Finds all (relevant) overlaps in the feature architecture

    :param max_overlap_percentage: overlap_threshold in percentage
    :param max_overlap: overlap threshold in amino acids
    :param overlap_region: region that is looked at
    :param features: feature information [start/stop]
    :return: overlap_map
    """

    overlap_map = [("START", overlap_region)]
    for i in range(0, len(overlap_region)):
        end = features[overlap_region[i]][3]
        length_i = features[overlap_region[i]][3] - features[overlap_region[i]][2]
        length_i = length_i * max_overlap_percentage
        overlap = []
        x = i + 1
        while x < len(overlap_region):
            length_x = features[overlap_region[x]][3] - features[overlap_region[x]][2]
            length_x = length_x * max_overlap_percentage
            overlap_size = end - features[overlap_region[x]][2]
            if overlap_size <= max_overlap and overlap_size <= length_i and overlap_size <= length_x:
                overlap.append(overlap_region[x])
            x += 1
        overlap.append("END")
        overlap_map.append((overlap_region[i], overlap))
    overlap_map.append(("END", []))
    overlap_map.reverse()
    return overlap_map



#!/bin/env python

#######################################################################
# Copyright (C) 2019 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################


def write_tsv_out(outpath, bidirectional, results):
    out = open(outpath + ".tsv", "w")
    outdict = {}
    out.write("Seed\tQuery\tScore(Forward/Reverse)\tMS(Forward/Reverse)\tPS(Forward/Reverse)\tCS(Forward/Reverse)"
              "\tLS(Forward/Reverse)\tMethod\n")
    for result in results[0]:
        outdict[result[0], result[1]] = (result[2], ("NA", "NA", "NA", "NA", "NA"), result[3])
    if bidirectional:
        for result in results[1]:
            outdict[result[1], result[0]] = (outdict[result[1], result[0]][0], result[2], outdict[result[1],
                                                                                                  result[0]][2])
    for pair in outdict:
        out.write(pair[0] + "\t" + pair[1] + "\t" + f"{outdict[pair][0][0]:.4}" + "/" +
                  f"{outdict[pair][1][0]:.4}" + "\t" + f"{outdict[pair][0][1]:.4}" + "/" +
                  f"{outdict[pair][1][1]:.4}" + "\t" + f"{outdict[pair][0][2]:.4}" + "/" +
                  f"{outdict[pair][1][2]:.4}" + "\t" + f"{outdict[pair][0][3]:.4}" + "/" +
                  f"{outdict[pair][1][3]:.4}" + "\t" + f"{outdict[pair][0][4]:.4}" + "/" +
                  f"{outdict[pair][1][4]:.4}" + "\t" + outdict[pair][2] + "\n")
    out.close()


def write_domain_out(seed_proteome, query_proteome, seed, query, weights, scale, seedpath, querypath, out, option):
    tools = option["input_linearized"] + option["input_normal"]
    uni_weight = None
    if option['reverse']:
        groupname = query
    else:
        groupname = seed
    if option["MS_uni"] and len(seedpath) > 0:
        uni_weight = round(1.0 / len(seedpath), 4)
    for tool in tools:
        for feature in seed_proteome[seed][tool]:
            if feature in seedpath:
                if option["MS_uni"]:
                    weight = uni_weight
                else:
                    weight = round(weights[feature] * scale, 4)
                for instance in seed_proteome[seed][tool][feature]["instance"]:
                    if (instance[0], instance[1]) in seedpath[feature]:
                        inpath = "Y"
                    else:
                        inpath = "N"
                    if option["reverse"]:
                        out.write(groupname + "#" + seed + "\t" + seed + "\t" + str(seed_proteome[seed]["length"]) +
                                  "\t" + feature + "\t" + str(instance[0]) + "\t" + str(instance[1]) + "\t" +
                                  str(weight) + "\t" + inpath + "\n")
                    else:
                        out.write(groupname + "#" + query + "\t" + seed + "\t" + str(seed_proteome[seed]["length"]) +
                                  "\t" + feature + "\t" + str(instance[0]) + "\t" + str(instance[1]) + "\t" +
                                  str(weight) + "\t" + inpath + "\n")
            else:
                for instance in seed_proteome[seed][tool][feature]["instance"]:
                    if option["reverse"]:
                        out.write(groupname + "#" + seed + "\t" + seed + "\t" + str(seed_proteome[seed]["length"]) +
                                  "\t" + feature + "\t" + str(instance[0]) + "\t" + str(instance[1]) + "\tNA\tN\n")
                    else:
                        out.write(groupname + "#" + query + "\t" + seed + "\t" + str(seed_proteome[seed]["length"]) +
                                  "\t" + feature + "\t" + str(instance[0]) + "\t" + str(instance[1]) + "\tNA\tN\n")
    if not seed == query:
        for tool in tools:
            for feature in query_proteome[query][tool]:
                if feature in seedpath:
                    if option["MS_uni"]:
                        weight = uni_weight
                    else:
                        weight = round(weights[feature] * scale, 4)
                else:
                    weight = "NA"
                if feature in querypath:
                    for instance in query_proteome[query][tool][feature]["instance"]:
                        if (instance[0], instance[1]) in querypath[feature]:
                            inpath = "Y"
                        else:
                            inpath = "N"
                        if option["reverse"]:
                            out.write(groupname + "#" + seed + "\t" + query + "\t" +
                                      str(query_proteome[query]["length"]) + "\t" + feature + "\t" + str(instance[0]) +
                                      "\t" + str(instance[1]) + "\t" + str(weight) + "\t" + inpath + "\n")
                        else:
                            out.write(groupname + "#" + query + "\t" + query + "\t" +
                                      str(query_proteome[query]["length"]) + "\t" + feature + "\t" + str(instance[0]) +
                                      "\t" + str(instance[1]) + "\t" + str(weight) + "\t" + inpath + "\n")
                else:
                    for instance in query_proteome[query][tool][feature]["instance"]:
                        if option["reverse"]:
                            out.write(groupname + "#" + seed + "\t" + query + "\t" +
                                      str(query_proteome[query]["length"]) + "\t" + feature + "\t" + str(instance[0]) +
                                      "\t" + str(instance[1]) + "\t" + str(weight) + "\tN\n")
                        else:
                            out.write(groupname + "#" + query + "\t" + query + "\t" +
                                      str(query_proteome[query]["length"]) + "\t" + feature + "\t" + str(instance[0]) +
                                      "\t" + str(instance[1]) + "\t" + str(weight) + "\tN\n")


def phyloprofile_out(outpath, bidirectional, mapping_file, results):
    with open(mapping_file) as infile:
        map = {}
        for line in infile.readlines():
            cells = line.rstrip("\n").split("\t")
            map[cells[0]] = cells[1]
    out = open(outpath + ".phyloprofile", "w")
    out.write("geneID\tncbiID\torthoID\tFAS_F\tFAS_B\n")
    outdict = {}

    for result in results[0]:
        outdict[result[0], result[1]] = (result[2][0], 0.0)
    if bidirectional:
        for result in results[1]:
            outdict[result[1], result[0]] = (outdict[result[1], result[0]][0], result[2][0])
    for pair in outdict:
        try:
            out.write(pair[0] + "\t" + map[pair[1]] + "\t" + pair[1] + "\t" + str(outdict[pair][0]) + "\t" +
                      str(outdict[pair][1]) + "\n")
        except KeyError:
            raise Exception(pair[1] + " not in mapping file")
    out.close()



#!/bin/env python

#######################################################################
# Copyright (C) 2019 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################


from greedyFAS.mainFAS.fasWeighting import w_weight_const_rescale


def sf_calc_score(path, protein, weights, search_features, query_features, seed_proteome, clan_dict, query_clans,
                  option):
    """Used to be main Scoring function, starts the functions for the scores and calculates the complete FAS score,
    only used during priority mode to make greedy decisions

    :param path: Path to score
    :param protein: protein id
    :param weights: feature weights
    :param query_features: all features to be linearized in the query protein
    :param search_features: all features to be linearized in the seed protein
    :param seed_proteome: dictionary that contains the feature architecture of all seed proteins
    :param clan_dict: dictionary that maps features to clans
    :param query_clans: (Pfam)-clans of the current query protein
    :param option: dictionary that contains the main option variables of FAS
    :return: score_ms, score_ps, score_cs, final_score
    """
    adjusted_weights = {}
    tmp_weight = {}
    if option["weight_const"]:
        adjusted_weights = w_weight_const_rescale(path, weights, search_features, False, option)
        tmp_weight = {}
        for i in adjusted_weights:
            tmp_weight[i] = weights[i]
            weights[i] = adjusted_weights[i]
    score_cs = round(sf_cs_score(path, clan_dict, query_clans, query_features), 4)
    tmp = sf_ms_score(path, protein, seed_proteome, query_features, option)
    score_ms = round(tmp[0], 4)
    score_ps = round(
        sf_ps_score(path, tmp[2], protein, query_features, seed_proteome, option), 4)
    final_score = (score_ms * option["score_weights"][0]) + (score_cs * option["score_weights"][1]) + (
        score_ps * option["score_weights"][2])
    if option["weight_const"]:
        for i in adjusted_weights:
            weights[i] = tmp_weight[i]
    return score_ms, score_ps, score_cs, final_score


def sf_entire_calc_score(path, query_path, weights, search_features, a_s_f, query_features, a_q_f, clan_dict, option):
    """Main Scoring function, starts the functions for the scores and calculates the complete FAS score

    :param path: Path to score
    :param query_path: Path to score (query)
    :param weights: feature weights
    :param query_features: all features to be linearized in the query protein
    :param search_features: all features to be linearized in the seed protein
    :param a_s_f: additional seed features [non linearized]
    :param a_q_f: additional query features [non linearized]
    :param clan_dict: dictionary that maps features to clans
    :param option: dictionary that contains the main option variables of FAS
    :return: score_ms, score_ps, score_cs, final_score, path_weight, common_feature, score_ls
    """
    adjusted_weights = {}
    tmp_weight = {}
    if option["weight_const"]:
        adjusted_weights = w_weight_const_rescale(path, weights, search_features, False, option)
        tmp_weight = {}
        for i in adjusted_weights:
            tmp_weight[i] = weights[i]
            weights[i] = adjusted_weights[i]
    score_cs = round(sf_entire_cs_score(path, query_path, query_features, clan_dict, search_features), 4)
    tmp = sf_entire_ms_score(path, query_path, search_features, a_s_f, query_features, a_q_f, weights, option)
    score_ms = round(tmp[0], 4)
    path_weight = tmp[3]
    common_feature = tmp[4]
    ps_tmp = sf_entire_ps_score(path, tmp[2], query_path, search_features, a_s_f, query_features, a_q_f, weights,
                                option)
    score_ps = round(ps_tmp[0], 4)
    score_ls = round(ps_tmp[1], 4)
    final_score = (score_ms * option["score_weights"][0]) + (score_cs * option["score_weights"][1]) + (
        score_ps * option["score_weights"][2])
    if option["weight_const"]:
        for i in adjusted_weights:
            weights[i] = tmp_weight[i]
    return score_ms, score_ps, score_cs, float(final_score), path_weight, common_feature, score_ls


def sf_cs_score(path, clan_dict, query_clans, features):
    """Calculates clan score

    :param path: Path to score
    :param clan_dict: dictionary that maps features to clans
    :param query_clans: Clans in the query
    :param features: features (seed or query) [dict]
    :return: score (PS)
    """

    counter = 0.0
    path_clans = {}
    score = 0.0

    # counting clans in path
    # path_clans: contains counts for clans in path
    for i in path:
        feature = features[i]

        if feature[0] in clan_dict:
            if clan_dict[feature[0]] in path_clans:
                path_clans[clan_dict[feature[0]]] += 1
            else:
                path_clans[clan_dict[feature[0]]] = 1
    for clan in path_clans:
        counter += 1.0
        if clan in query_clans:
            score += float(path_clans[clan] * query_clans[clan]) / float(
                max(path_clans[clan], query_clans[clan]) * max(path_clans[clan], query_clans[clan]))
    if counter == 0:
        score = 0.0
    else:
        score = score / counter
    return float(score)


def sf_entire_cs_score(path, query_path, query_features, clan_dict, search_features):
    """calculates clan score

    :param query_path:
    :param path: Path to score
    :param query_features: all features to be linearized in the query protein
    :param search_features: all features to be linearized in the seed protein
    :param clan_dict: dictionary that maps features to clans
    :return: score (PS)
    """

    counter = 0.0
    s_clans = {}  # clans from path
    q_clans = {}  # clans from query_path

    score = 0.0
    for i in path:
        if i in search_features:
            feature = search_features[i]
            if feature[0] in clan_dict:
                if clan_dict[feature[0]] in s_clans:
                    s_clans[clan_dict[feature[0]]] += 1
                else:
                    s_clans[clan_dict[feature[0]]] = 1
    for i in query_path:
        if i in query_features:
            feature = query_features[i]
            if feature[0] in clan_dict:
                if clan_dict[feature[0]] in q_clans:
                    q_clans[clan_dict[feature[0]]] += 1
                else:
                    q_clans[clan_dict[feature[0]]] = 1

    for clan in s_clans:
        counter += 1.0
        if clan in q_clans:
            score += float(s_clans[clan] * q_clans[clan]) / float(
                max(s_clans[clan], q_clans[clan]) * max(s_clans[clan], q_clans[clan]))
    if counter == 0:
        score = 0.0
    else:
        score = score / counter
    return float(score)


def sf_ms_score(path, protein, proteome, features, option):
    """Calculates multiplicity score, only used for priority mode now

    :param path : Path to score
    :param protein : protein id
    :param proteome : seed_proteome is a dictionary that contains the feature architecture of all seed proteins
    :param features : features (seed or query) [dict]
    :param option : specifies the behavior of the MS calculation
    :return: final_score (MS), search_domains, scale
    """
    domains = {}
    scale = 0
    scores = []
    final_score = 0.0
    tools = option["input_linearized"] + option["input_normal"]
    for i in path:
        feature = features[i]
        if feature[0] in domains:
            domains[feature[0]] += 1
        else:
            domains[feature[0]] = 1
    for feature in domains:
        scale += 1
        p_score = 0.0
        for tool in tools:
            if feature in proteome[protein][tool]:
                e_feature = False
                try:
                    if proteome[protein][tool][feature]["evalue"] <= option["eFeature"]:
                        e_feature = True
                except TypeError:
                    e_feature = True
                if e_feature:
                    s_length = 0
                    for instance in proteome[protein][tool][feature]["instance"]:
                        e_instance = False
                        try:
                            if instance[2] <= option["eInstance"]:
                                s_length += 1
                        except TypeError:
                            s_length += 1
                    p_score = min(float(domains[feature] * s_length) / float(domains[feature] * domains[feature]), 1.0)
        scores.append((feature, p_score))
    if scale > 0:
        scale = 1.0 / float(scale)
    for score in scores:
        final_score += score[1] * scale
    return float(final_score), domains, scale


def sf_entire_ms_score(path, query_path, search_features, a_s_f, query_features, a_q_f, weights, option):
    """Calculates multiplicity score

    :param path: Path to score
    :param query_path: Path to score (query)
    :param weights: feature weights
    :param query_features: all features to be linearized in the query protein
    :param search_features: all features to be linearized in the seed protein
    :param a_s_f: additional seed features [non linearized]
    :param a_q_f: additional query features [non linearized]
    :param option: dictionary that contains the main option variables of FAS
    :return: final_score (MS), search_domains, scale, final_weight, common_feature
    """
    search_domains = {}
    query_domains = {}
    scale = 0
    scores = []
    final_score = 0.0
    final_weight = 0.0
    main_features_s = {}
    main_features_q = []
    common_feature = False

    for i in path:
        if i in search_features:
            feature = search_features[i]
            main_features_s[feature[0]] = True
        else:
            feature = a_s_f[i]

        if feature[0] in search_domains:
            search_domains[feature[0]] += 1
        else:
            search_domains[feature[0]] = 1

    for i in query_path:
        if i in query_features:
            feature = query_features[i]
            main_features_q.append(feature[0])
        else:
            feature = a_q_f[i]
        if feature[0] in query_domains:
            query_domains[feature[0]] += 1
        else:
            query_domains[feature[0]] = 1

    # check for common features
    for y in main_features_q:
        try:
            if main_features_s[y]:
                common_feature = True
        except KeyError:
            pass
    for feature in search_domains:
        if option["MS_uni"] == 0:
            scale += weights[feature]
        else:
            scale += 1
        if feature in query_domains:
            s_length = query_domains[feature]
            p_score = min(float(search_domains[feature] * s_length) /
                          float(search_domains[feature] * search_domains[feature]), 1.0)
            scores.append((feature, p_score))
        else:
            scores.append((feature, 0.0))
    if scale > 0:
        scale = 1.0 / float(scale)
    for score in scores:
        if option["MS_uni"] == 0:
            final_score += score[1] * scale * weights[score[0]]
            final_weight += weights[score[0]]
        else:
            final_score += score[1] * scale
    return float(final_score), search_domains, scale, final_weight, common_feature


def sf_ps_score(path, scale, protein, features, seed_proteome, option):
    """Calculates positional score

    :param path: Path to score
    :param scale: contains scaling factor for each weight or number of feature types for uniform weighting
    :param protein: protein id
    :param features: features (seed or query) [dict]
    :param seed_proteome: dictionary that contains the feature architecture of all seed proteins
    :return: final_score (PS)
    """
    count = {}
    final_score = 0.0
    scores = {}
    tools = option["input_linearized"] + option["input_normal"]
    best_match = 0.0
    for i in path:
        feature = features[i]
        for tool in tools:
            if feature[0] in seed_proteome[protein][tool]:
                e_feature = False
                try:
                    if seed_proteome[protein][tool][feature[0]]["evalue"] <= option["eFeature"]:
                        e_feature = True
                except TypeError:
                    e_feature = True
                if e_feature:
                    best_match = 0.0
                    if not feature[0] in scores:
                        scores[feature[0]] = 0.0
                        count[feature[0]] = 0
                    for instance in seed_proteome[protein][tool][feature[0]]["instance"]:
                        e_instance = False
                        try:
                            if instance[2] <= option["eInstance"]:
                                e_instance = True
                        except TypeError:
                            e_instance = True
                        if e_instance:
                            pos = (float(instance[1]) + float(instance[2])) / 2.0 / float(
                                seed_proteome[protein]["length"])
                            match = 1.0 - float(abs(feature[1]) - pos)
                            if best_match < match:
                                best_match = match
                scores[feature[0]] += best_match
                count[feature[0]] += 1

    for f_score in scores:
        final_score += scores[f_score] / count[f_score] * scale
    return float(final_score)


def sf_entire_ps_score(path, scale, query_path, search_features, a_s_f, query_features, a_q_f, weights, option):
    """Calculates positional score

    :param path: Path to score
    :param scale: contains scaling factor for each weight or number of feature types for uniform weighting
    :param query_path: Path to score (query)
    :param weights: feature weights
    :param query_features: all features to be linearized in the query protein
    :param search_features: all features to be linearized in the seed protein
    :param a_s_f: additional seed features [non linearized]
    :param a_q_f: additional query features [non linearized]
    :param option: dictionary that contains the main option variables of FAS
    :return: final_score (PS), final_ls_score
    """
    count = {}
    final_score = 0.0
    final_ls_score = 0.0
    scores = {}
    ls_scores = {}
    local_query_protein = {}
    ls = 0.0
    # get current features from query path
    for i in query_path:
        if i in query_features:
            feature = query_features[i]
        else:
            feature = a_q_f[i]
        if feature[0] in local_query_protein:
            length = feature[3] - feature[2] + 1.0
            local_query_protein[feature[0]].append((feature[1], length))
        else:
            local_query_protein[feature[0]] = [(feature[1], feature[3] - feature[2] + 1.0)]

    # compare features in path with features form query path
    for i in path:
        if i in search_features:
            feature = search_features[i]
        else:
            feature = a_s_f[i]
        if feature[0] in local_query_protein:
            best_match = 0.0
            if not feature[0] in scores:
                count[feature[0]] = 0
                scores[feature[0]] = 0.0
                ls_scores[feature[0]] = 0.0
            for instance in local_query_protein[feature[0]]:
                match = 1.0 - float(abs(feature[1] - instance[0]))
                if best_match < match:
                    best_match = match
                    if instance[1] <= (feature[3] - feature[2]):
                        ls = float(instance[1]) / float(feature[3] - feature[2] + 1.0)
                    else:
                        ls = float(feature[3] - feature[2] + 1.0) / float(instance[1])

            scores[feature[0]] += best_match
            ls_scores[feature[0]] += ls
            count[feature[0]] += 1

    for f_score in scores:
        if option["MS_uni"] == 0:
            final_score += scores[f_score] / count[f_score] * scale * weights[f_score]
            final_ls_score += ls_scores[f_score] / count[f_score] * scale * weights[f_score]
        else:
            final_score += scores[f_score] / count[f_score] * scale
            final_ls_score += ls_scores[f_score] / count[f_score] * scale

    return float(final_score), float(final_ls_score)



#!/bin/env python

#######################################################################
# Copyright (C) 2019 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################

import xml.etree.ElementTree as ElTre
import os
import json


def xmlreader(path, mode, tool, assess, proteome, protein_lengths, clan_dict, option):
    """
    Input function,
    read input-files for seed, query and reference

    :param path: path to input file
    :param mode: 0 (seed), 1 (query), 2 (reference)
    :param tool: tool/database name (pfam, seg, ...)
    :param assess: e-value assess
    :param proteome: proteome var to write in [dict]
    :param protein_lengths: dictionary that contains the length of each protein in the seed and query
    :param clan_dict: dictionary that maps features to clans
    :param option: dictionary that contains the main option variables of FAS
    :return: proteome, protein_lengths, clan_dict
    """
    start = 0
    if os.path.exists(path):
        try:
            xmltree = ElTre.parse(path)
        except ElTre.ParseError:
            raise Exception(path + ' is not a legit xml file')
        root = xmltree.getroot()
        for protein in root:
            p_id = protein.attrib["id"]
            plength = protein.attrib["length"]
            # set up of protein IDs, differentiate between proteins from different files
            if mode == 1:
                if(not option["query_id"]) or p_id in option["query_id"]:
                    protein_lengths["query_" + str(p_id)] = float(plength)
            elif mode == 0:
                if (not option["seed_id"]) or p_id in option["seed_id"]:
                    protein_lengths["seed_" + str(p_id)] = float(plength)
            elif mode == 2:
                protein_lengths[p_id] = float(plength)
            # set up of datastructure to store annotations
            if mode == 2 or (mode == 1 and ((not option["query_id"]) or p_id in option["query_id"])) or (
               mode == 0 and ((not option["seed_id"]) or p_id in option["seed_id"])):
                if not (p_id in proteome):
                    proteome[p_id] = {}

                for feature in protein:
                    if len(feature) > 0:
                        ftype = tool + "_" + feature.attrib["type"]
                        feat_eval = 'NULL'

                        # evalue check: family/ftype based
                        if 'evalue' in feature.attrib and float(feature.attrib["evalue"]) > option["efilter"]:
                            # skip current ftype and continue with the next one
                            continue
                        else:
                            # keep feature type bases evalue
                            if 'evalue' in feature.attrib:
                                feat_eval = float(feature.attrib["evalue"])

                            if 'clan' in feature.attrib:
                                fclan = feature.attrib["clan"]
                            else:
                                fclan = "NO_CLAN"
                            proteome[p_id][ftype] = []
                            proteome[p_id][ftype].append(assess)
                            proteome[p_id][ftype].append(feat_eval)

                            i = 0
                            # counting appended instances
                            inst_count = 0
                            for instance in feature:
                                inst_eval = 'NULL'
                                # XMLcase 1: feature instance contains evalue information (XML field inst_eval)
                                if 'inst_eval' in instance.attrib:
                                    # print tool + " instance evalue: "+ str(instance.attrib)
                                    inst_eval = float(instance.attrib["inst_eval"])
                                    start = int(instance.attrib["start"])
                                    end = int(instance.attrib["end"])

                                    if inst_eval <= option["inst_efilter"]:
                                        proteome[p_id][ftype].append((inst_eval, start, end))
                                        inst_count += 1

                                # XMLcase 2: feature instance contains NO evalue information (XML field inst_eval)
                                else:
                                    # NO instance based evalue information --> no instances can be rejected:
                                    # set inst_count = 1
                                    inst_count = 1
                                    if len(instance.attrib) == 2:
                                        start = int(instance.attrib["start"])
                                        end = int(instance.attrib["end"])
                                        proteome[p_id][ftype].append((inst_eval, start, end))

                                    else:
                                        if i == 0:
                                            start = int(instance.attrib["start"])
                                            i = 1
                                        else:
                                            end = int(instance.attrib["end"])
                                            proteome[p_id][ftype].append((inst_eval, start, end))
                                            i = 0
                            # any instance appended?
                            if inst_count < 1:
                                # delete feature type
                                proteome[p_id].pop(ftype)
                            if ftype not in clan_dict:
                                clan_dict[ftype] = fclan
    else:
        raise Exception(path + " does not exist")
    return proteome, protein_lengths, clan_dict


def featuretypes(path, option):
    """
    Input function,
    reads the tools/featuretypes input-file and stores information in option

    :param path: path to input-file
    :param option: dictionary that contains the main option variables of FAS
    :return: option
    """
    option["input_linearized"] = []
    option["input_normal"] = []
    ifile = open(path, "r")
    lines = ifile.readlines()
    mode = "NULL"
    for line in lines:
        tmp = line.rstrip("\n").lower()
        if tmp == "#linearized":
            mode = "lin"
        elif tmp == "#normal":
            mode = "nor"
        elif tmp == "#checked":
            mode = "ignore"
        elif mode == "NULL":
            raise Exception(path + " is not a valid input file")
        elif mode == "lin" and len(tmp) > 0:
            option["input_linearized"].append(tmp)
        elif mode == "nor" and len(tmp) > 0:
            option["input_normal"].append(tmp)
    ifile.close()
    return option


def constraints_in(path):
    """
    Input function,
    reads the constraints file

    :param path: path to input-file
    :return: constraints
    """
    constraints = {}
    cfile = open(path, "r")
    lines = cfile.readlines()
    i = 1
    if lines[0][0] == "#":
        while lines[i][0] != "#":
            split = (lines[i].rstrip("\n")).split(" ")
            if split[1] != "N":
                constraints[split[0]] = float(split[1])
            i += 1
    else:
        raise Exception(path + " might be in the wrong format. Please see the sample file in config directory.")
    if lines[i][0] == "#":
        i += 1
        while i < len(lines):
            split = (lines[i].rstrip("\n")).split(" ")
            constraints[split[0]] = float(split[1])
            i += 1
    cfile.close()
    return constraints


def read_pairwise(path):
    with open(path, 'r') as infile:
        pairwise = []
        line = infile.readline()
        while line:
            pairwise.append(line.rstrip('\n').split('\t'))
            line = infile.readline()
    return pairwise


def read_json(path):
    with open(path, 'r') as infile:
        in_dict = json.loads(infile.read())
    return in_dict



#!/bin/env python

#######################################################################
# Copyright (C) 2020 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################


import os
import argparse
import multiprocessing as mp
from greedyFAS.annoFAS import annoFAS
from greedyFAS.annoFAS import annoModules
from greedyFAS.mainFAS import fasInput, greedyFAS


def get_options():
    version = '1.4.13'
    parser = argparse.ArgumentParser(description='You are running FAS version ' + str(version) + '.',
                                     epilog="For more information on certain options, please refer to the wiki pages "
                                            "on github: https://github.com/BIONF/FAS/wiki")
    required = parser.add_argument_group('required arguments')
    general = parser.add_argument_group('general arguments')
    inargs = parser.add_argument_group('input arguments')
    outargs = parser.add_argument_group('output arguments')
    annotation = parser.add_argument_group('annotation arguments')
    weighting = parser.add_argument_group('weighting arguments')
    thresholds = parser.add_argument_group('threshold arguments')
    obscure = parser.add_argument_group('obscure arguments')
    parser.add_argument('--version', action='version', version=str(version))
    required.add_argument("-s", "--seed", default=None, type=str, required=True,
                          help="path to seed protein fasta file")
    required.add_argument("-q", "--query", default=None, type=str, required=True,
                          help="path to query protein fasta file")
    required.add_argument("-a", "--annotation_dir", default=None, type=str, required=True,
                          help='working directory, all annotations are be stored here')
    required.add_argument("-o", "--out_dir", default=None, type=str, required=True,
                          help="output directory, all outputfiles will be stored here")
    general.add_argument("--bidirectional", action="store_true",
                         help="calculate both scoring directions")
    general.add_argument('--cpus', help='number of cores', type=int, action='store', default=0)
    annotation.add_argument('--force', help='Force override annotations', action='store_true')
    annotation.add_argument("-f", "--eFeature", default="0.001", type=float,
                            help="eValue cutoff for PFAM/SMART domain, applied during annotation but also during "
                                 "calculation")
    annotation.add_argument("-i", "--eInstance", default="0.01", type=float,
                            help="eValue cutoff for PFAM/SMART instances, applied during annotation but also during "
                                 "calculation")
    annotation.add_argument('--eFlps', help='eValue cutoff for fLPS', action='store', default=0.0000001, type=float)
    annotation.add_argument('--org', help='Organism of input sequence(s) for SignalP search',
                            choices=['euk', 'gram+', 'gram-'], action='store', default='euk', type=str)
    annotation.add_argument("--toolPath", dest="toolPath", default='', type=str,
                            help="Path to Annotion tool directory created with setupFAS")
    weighting.add_argument("-r", "--ref_proteome", default=None, type=str,
                           help="Path to a reference proteome which can be used for the weighting of features, "
                                "by default there is no reference proteome used, the weighting will be uniform")
    weighting.add_argument("--ref_2", dest="ref_2", default=None, type=str,
                           help="Give a second reference for bidirectional mode, does not do anything if bidirectional "
                                "mode is not active or if no main reference was given")
    weighting.add_argument("-g", "--weight_correction", default="loge", type=str,
                           help="Function applied to the frequency of feature types during weighting, options are "
                                "linear(no function), loge(natural logarithm[Default]), log10(base-10 logarithm), "
                                "root4(4th root) and root8(8th root).")
    weighting.add_argument("-x", "--weight_constraints", default=None, type=str,
                           help="Apply weight constraints via constraints file, by default there are no constraints. "
                                "Please look at the FAS wiki pages for templates for the constraints file")
    inargs.add_argument("--query_id", default=None, nargs='*', type=str,
                        help="Choose specific proteins (ids divided by spaces) from the query input for calculation, "
                             "by default this is off (all proteins are used for calculation)")
    inargs.add_argument("--seed_id", default=None, nargs='*', type=str,
                        help="Choose specific proteins (ids divided by spaces) from the seed input for calculation, "
                             "by default this is off (all proteins are used for calculation)")
    inargs.add_argument("--pairwise", dest="pairwise", default=None, type=str,
                        help="deactivate all against all comparison, needs a pairing file with the ids that should be"
                             " compared (one pair per line tab seperated), please look at the FAS wiki pages for "
                             "templates")
    inargs.add_argument("-d", "--featuretypes", default=None, type=str,
                        help="inputfile that contains the tools/databases used to predict features. Please look at the "
                             "FAS wiki pages for templates of the the featuretypes input file")
    inargs.add_argument("--extra_annotation", default=None, nargs='*', type=str,
                        help="give naming conventions for extra annotation files, eg. if the file name for the seed is "
                             "someseed.json than the extra annotation files should be named someseed_"
                             "[EXTRA_ANNOTATION].json, these extra files need to exist for seed, query and the "
                             "references (if given)")
    outargs.add_argument("-n", "--out_name", default=None, type=str,
                         help="name for outputfiles, if none is given the name will be created from the seed and "
                              "query names")
    outargs.add_argument("--raw", dest="raw", action="store_true",
                         help="print FAS scores to terminal")
    outargs.add_argument("--tsv", dest="silent", action="store_true",
                         help="deactivates creation of the tsv output, either use together with --raw or "
                              "--phyloprofile")
    outargs.add_argument("--phyloprofile", dest="phyloprofile", default=None, type=str,
                         help="activate phyloprofile output, needs mapping file for all query proteins")
    outargs.add_argument("--domain", dest="domain", action="store_false",
                         help="deactivate .domains output")
    thresholds.add_argument("-c", "--max_overlap", dest="max_overlap", default=0, type=int,
                            help="maximum size overlap allowed, default is 0 amino acids")
    thresholds.add_argument("--max_overlap_percentage", dest="max_overlap_percentage", default=0.4, type=float,
                            help="defines how much percent of a feature the overlap is allowed to cover, default "
                                 "is 0.4 (40%%)")
    thresholds.add_argument("-t", "--priority_threshold", type=int, default=30,
                            help="Change to define the feature number threshold for activating priority mode in the "
                                 "path evaluation. default=30")
    thresholds.add_argument("-m", "--max_cardinality", default=500, type=int,
                            help="Change to define the threshold for the maximal cardinality (number) of feature paths "
                                 "in a graph. If max. cardinality is exceeded the priority mode will be used to for "
                                 "the path evaluation. default=500")
    obscure.add_argument("--priority_mode", action='store_false',
                         help="deactivates the greedy strategy priority mode for larger architectures, NOT RECOMMENDED")
    obscure.add_argument("--timelimit", default=3600, type=int,
                         help="Sets a soft time limit in seconds for the calculation between a pair of proteins,"
                              "This limit does not necessarily represent the actual runtime. It only stops the "
                              "exhaustive strategy through the architecture and activate priority mode. "
                              "This option is only relevant if the option [--priority_mode] is set")
    obscure.add_argument("-w", "--score_weights", nargs=3, default=[0.7, 0.0, 0.3], type=float,
                         help="Defines how the three scores MS, CS and PS are weighted, takes three float arguments, "
                              "sum should be 1.0, the default is 0.7, 0.0, 0.3")
    args = parser.parse_args()
    return args


def anno(annojobs, args, toolpath):
    eflps = args.eFlps
    signalporg = args.org
    efeature = args.eFeature
    einstance = args.eInstance
    hmmcores = 1
    cpus = args.cpus
    if cpus == 0:
        cpus = mp.cpu_count()-1
    for annojob in annojobs:
        name = ''.join(annojob.split('/')[-1].split('.')[:-1])
        outpath = os.path.abspath(args.annotation_dir)
        annotate = True
        seqfile = annojob
        if os.path.isdir(args.annotation_dir + '/' + name + '.json'):
            print('Annotation for "' + name + '" already exists.')
            if args.force:
                print('Overwriting...')
                annoModules.checkFileExist(seqfile)
            else:
                print('Skipping annotation...')
                annotate = False
        else:
            annoModules.checkFileExist(seqfile)
            print('Check annotation for "' + name + '"...')
        if annotate:
            annoFAS.runAnnoFas(
                [seqfile, outpath, toolpath, args.force, name, eflps, signalporg, efeature, einstance, hmmcores, '',
                 '', '', cpus])


def fas(args, toolpath):
    option_dict = {
                   "weight_const": False, "seed_id": args.seed_id, "query_id": args.query_id,
                   "priority_mode": args.priority_mode, "priority_threshold": args.priority_threshold,
                   "max_cardinality": args.max_cardinality, "cores": int(args.cpus), "raw": args.raw,
                   "bidirectional": args.bidirectional, "max_overlap": args.max_overlap,
                   "timelimit": args.timelimit, "phyloprofile": args.phyloprofile, "score_weights": [],
                    "output": args.silent, "max_overlap_percentage": 0.0, "domain": args.domain, "pairwise": None,
                    "eInstance": args.eInstance, "eFeature": args.eFeature, "progress": True
                   }
    seedname = ''.join(args.seed.split('/')[-1].split('.')[:-1])
    option_dict["p_path"] = [args.annotation_dir + '/' + seedname + '.json']
    queryname = ''.join(args.query.split('/')[-1].split('.')[:-1])
    option_dict["s_path"] = [args.annotation_dir + '/' + queryname + '.json']
    if args.ref_proteome:
        name = ''.join(args.ref_proteome.split('/')[-1].split('.')[:-1])
        option_dict["ref_proteome"] = [args.annotation_dir + '/' + name + '.json']
    else:
        option_dict["ref_proteome"] = None
    if args.raw:
        option_dict["progress"] = False
    if option_dict["cores"] == 0:
        option_dict["cores"] = mp.cpu_count()-1
    if args.ref_2:
        r2name = ''.join(args.ref_2.split('/')[-1].split('.')[:-1])
        option_dict["ref_2"] = [args.annotation_dir + '/' + r2name + '.json']
    else:
        option_dict["ref_2"] = None
    if args.extra_annotation:
        for i in args.extra_annotation:
            option_dict["p_path"].append(args.annotation_dir + '/' + i + '.json')
            option_dict["s_path"].append(args.annotation_dir + '/' + i + '.json')
            if args.ref_proteome:
                option_dict["ref_proteome"].append(args.annotation_dir + '/' + i + '.json')
            if args.ref_2:
                option_dict["ref_2"].append(args.annotation_dir + '/' + i + '.json')
    if not option_dict["ref_proteome"]:
        option_dict["MS_uni"] = 1
        option_dict["weight_correction"] = None
    try:
        test = 0.0
        for weight in args.score_weights:
            option_dict["score_weights"].append(weight)
            test += float(weight)
        if test != 1.0:
            raise Exception("The sum of all values in -w [--score_weights] should be 1.0")
        option_dict["score_weights"] = tuple(option_dict["score_weights"])
    except ValueError:
        raise Exception(str(args.score_weights) + " is not a valid input for -w [--score_weights]")
    if not option_dict["ref_proteome"]:
        option_dict["MS_uni"] = 1
        option_dict["weight_correction"] = None
    else:
        option_dict["MS_uni"] = 0
        if args.weight_correction == "log10":
            option_dict["weight_correction"] = "log10"
        elif args.weight_correction == "loge":
            option_dict["weight_correction"] = "loge"
        elif args.weight_correction == "root4":
            option_dict["weight_correction"] = "root4"
        elif args.weight_correction == "root8":
            option_dict["weight_correction"] = "root8"
        elif args.weight_correction == "linear":
            option_dict["weight_correction"] = None
        else:
            raise Exception(str(args.weightcorrection) + "is not a valid input for -g [--weightcorrection]")
    if args.ref_proteome and args.weight_constraints:
        option_dict["weight_const"] = True
    elif args.weight_constraints:
        raise Exception("[--weight_constraints] only works with a reference proteome")
    if 0.0 <= float(args.max_overlap_percentage) <= 1.0:
        option_dict["max_overlap_percentage"] = float(args.max_overlap_percentage)
    else:
        raise Exception("[--max_overlap_percentage] should be between 0.0 and 1.0")
    if args.out_name:
        option_dict['outpath'] = args.out_dir.rstrip('/') + '/' + args.out_name
    else:
        option_dict['outpath'] = args.out_dir.rstrip('/') + '/' + seedname + '_' + queryname
    if args.featuretypes is not None:
        option_dict = fasInput.featuretypes(args.featuretypes, option_dict)
    else:
        option_dict = fasInput.featuretypes(toolpath + '/' + 'annoTools.txt', option_dict)

    if args.pairwise:
        option_dict["pairwise"] = fasInput.read_pairwise(args.pairwise)
    else:
        option_dict["pairwise"] = None
    print('Calculating FAS score...')
    greedyFAS.fc_start(option_dict)
    print('done!')


def main():
    args = get_options()
    toolpath = args.toolPath
    if toolpath == '':
        pathconfigfile = os.path.realpath(__file__).replace('calcFAS.py', 'pathconfig.txt')
        with open(pathconfigfile) as f:
            toolpath = f.readline().strip()
    else:
        toolpath = os.path.abspath(args.toolPath)
    if not os.path.isdir(args.annotation_dir):
        os.mkdir(args.annotation_dir)
    if not os.path.isdir(args.out_dir):
        os.mkdir(args.out_dir)
    annojobs = [args.seed]
    if args.query not in annojobs:
        annojobs.append(args.query)
    if args.ref_proteome and args.ref_proteome not in annojobs:
        annojobs.append(args.ref_proteome)
    if args.ref_2 and args.ref_2 not in annojobs:
        annojobs.append(args.ref_2)
    anno(annojobs, args, toolpath)
    fas(args, toolpath)


if __name__ == '__main__':
    main()



#!/bin/env python

#######################################################################
# Copyright (C) 2020 Julian Dosch
#
# This file is part of FAS.
#
#  FAS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  FAS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with FAS.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################


import argparse
import json


def read_infile_single(ignore_lines, feature_columns, tool_name, path):
    infile = open(path, 'r')
    for i in range(ignore_lines):
        infile.readline()
    line = infile.readline()
    feature_dict = {}
    while line:
        cells = line.rstrip('\n').split('\t')
        p_id, length = cells[feature_columns[0]], int(cells[feature_columns[1]])
        start, stop = int(cells[feature_columns[3]]), int(cells[feature_columns[4]])
        f_id = tool_name + '_' + cells[feature_columns[2]]
        if p_id not in feature_dict:
            feature_dict[p_id] = {'length': length, tool_name: {}}
        if f_id not in feature_dict[p_id][tool_name]:
            feature_dict[p_id][tool_name][f_id] = {'evalue': 'NA', 'instance': []}
        feature_dict[p_id][tool_name][f_id]['instance'].append([start, stop, 'NA'])
        line = infile.readline()
    return feature_dict


def read_infile_multiple(ignore_lines, feature_columns, tool_names, tool_column, path):
    infile = open(path, 'r')
    for i in range(ignore_lines):
        infile.readline()
    line = infile.readline()
    feature_dict = {}
    while line:
        cells = line.rstrip('\n').split('\t')
        p_id, length = cells[feature_columns[0]], int(cells[feature_columns[1]])
        start, stop = int(cells[feature_columns[3]]), int(cells[feature_columns[4]])
        tool_name = cells[tool_column].lower()
        f_id = tool_name + '_' + cells[feature_columns[2]]
        if tool_name not in tool_names:
            raise Exception('tsv file contains the tool: "' + tool_name + '", which was not given in '
                                                                          '[-t] [--tool_names]')
        if p_id not in feature_dict:
            feature_dict[p_id] = {'length': length}
            for tool in tool_names:
                feature_dict[p_id][tool.lower()] = {}
        if f_id not in feature_dict[p_id][tool_name]:
            feature_dict[p_id][tool_name][f_id] = {'evalue': 'NA', 'instance': []}
        feature_dict[p_id][tool_name][f_id]['instance'].append([start, stop, 'NA'])
        line = infile.readline()
    return feature_dict


def count_features(feature_dict):
    count = {}
    for p_id in feature_dict:
        for tool in feature_dict[p_id]:
            if not tool == 'length':
                for f_id in feature_dict[p_id][tool]:
                    if f_id not in count:
                        count[f_id] = 0
                    count[f_id] += len(feature_dict[p_id][tool][f_id]['instance'])
    return count


def write_json(dict2save, path):
    jsonOut = json.dumps(dict2save, ensure_ascii=False)
    f = open(path, 'w')
    f.write(jsonOut)
    f.close()


def create_json(args):
    if len(args.tool_names) > 1 and args.tool_column:
        tool_names = []
        for i in args.tool_names:
            tool_names.append(str(i).lower())
        feature_dict = read_infile_multiple(args.ignore_lines, args.feature_columns, tool_names,
                                            args.tool_column, args.input)
    elif len(args.tool_names) > 1:
        raise Exception("Multiple tool names given. Please select the column containing tool names with "
                        "[-c][--tool_column]")
    else:
        feature_dict = read_infile_single(args.ignore_lines, args.feature_columns, args.tool_names[0].lower(),
                                          args.input)
    count = count_features(feature_dict)
    out_dict = {'feature': feature_dict, 'clan': {}, 'count': count}
    write_json(out_dict, args.output)


def main():
    parser = argparse.ArgumentParser(description="Parses .tsv formatted files into the json format used by FAS")
    required = parser.add_argument_group('required arguments')
    optional = parser.add_argument_group('optional arguments')
    required.add_argument("-i", "--input", type=str, default=None, required=True,
                          help="Path to input file, needs to be in tsv format")
    required.add_argument("-o", "--output", type=str, default=None, required=True,
                          help="Path to output file, should be a .json file")
    optional.add_argument("--ignore_lines", type=int, help="skip the first n lines (for example headers)", default=0)
    required.add_argument("-t", "--tool_names", type=str, nargs='+', required=True,
                          help="name of the annotation tool(s) in the tsv file, if multiple are given, the argument "
                               "--tool_name must point to the column containing the tool name and the tool names must "
                               "be identical to the ones in the tsv file")
    required.add_argument("-f", "--feature_columns", type=int, nargs=5, required=True,
                          help="needs 5 integer values that point to the columns that contain: "
                               "(1) the protein id, (2) the protein length, (3) the feature id, "
                               "(4) the start position of the feature, (5) the end position of the feature; the column "
                               "indices start at 0 so the first column has index 0, the second index 1, etc.")
    optional.add_argument("-c", "--tool_column", type=int, default=None,
                          help="The index (integer) of column that contains the annotation tool name, only necessary "
                               "if the tsv file contains multiple annotation tools like the tsv output of InterPro")
    args = parser.parse_args()
    create_json(args)
